# Functions that implement "Game Logic".  (Uhm, really, this is a place
# where I stick complicated functions, functions that aren't just 
# "plumbing", functions that I want to be able to test w/out AppEngine
# infrastructure.)

SUGGESTED_ORDER = [
    "example",

    "offbeats",
    "teuton",
    "crossword",
    "rebus",
    "sudoku",
    "onestepbeyond",
    "flags",

    "richmond",
    "march",
    "checkmate",
    "dualtone",

    "meta"
]

META_ORDER = [
    "example",
    "rebus",
    "teuton",
    "sudoku",
    "onestepbeyond",
    "crossword",
    "checkmate",
    "dualtone",
    "flags",
    "richmond",
    "march",
    "offbeats",
    "meta",
]

ACT_DATA = {
  "example" : {
    "instructions": "/s/example.html",
    "solution": "response",
    "solved": "SOLVED",
    "hours": [],
    "hints": [
      (1, "Did you read the instructions?"),
      (2, "The instructions tell you what the answer is."),
      (3, "The answer starts with the letter 'R'."),
      (4, "The answer rhymes with 'ESPONSE'"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "response": ("", 0),
      "repose"  : ("Almost, but missing an S and an N", 15),
      "esponse" : ("Almost, but missing an R", 20),
      },
  },
  "flags" : {
    "instructions": "/s/flags.html",
    "solution": "sunlight",
    "solved": "SOL\\/ED",
    "hours": [],
    "hints": [
      (1, "Do you know about semaphore signal flags?"),
      (30, "Did you see the racetrack map at the top of the puzzle?"),
      (40, "The racetrack map has angles that are multiples of 45 degrees."),
      (50, "Can you use semaphore letters to draw other racetracks?"),
      (60, "If you draw a semaphore J by a semaphore R, you can connect the horizontal arm of the J to the R.  By drawing more semaphore letters and connecting them this way, you can draw shapes."),
      (70, "The things you draw look like the outlines of letters."),
      (80, "The fifth sequence makes a skinny rectangle, like a capital letter I."),
      (90, "The letters spell a word."),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "sunlight": ("", 0),
      "straight": ("Wow, you're incredibly close, just three letters off.", 50),
      "semaphore": ("Yep, semaphore would be a good thing to use here.", 5),
      }
  },
  "richmond" : {
    "instructions": "/s/richmond.html",
    "solution": "bipolar",
    "solved": """SO<span style="color: green;">L</span>V<span style="color: blue;">E</span>D""",
    "hours": [0, 3, 5, 9, 10, 17, 24], 
    "hints": [
      (0,  "Go to that neighborhood and look around. Or if you're not close to San Francisco, guess SCOUT."),
      (100,"Go to the corner of 4th Ave and Cornwall (in the purple area of your map) and look for something purple."),
      (110, "Look for businesses with colorful names."),
      (140, 'For "Purple Crayon", write "CRAYON" on the blanks in line with the purple circle.'),
      (150, "With the right letters in the blanks, you should see a word in a column of blanks."),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response" : {
      "bipolar": ("", 0),
      "eclair": ("You solved an old version of this puzzle.  Try BIPOLAR.", 150),
      "sausalito": ("You solved an old version of this puzzle.  Try BIPOLAR.", 150),
      "scout": ("You can solve this puzzle with Google Street View.", 1),
      "whiteginger": ("You have the right general idea, but you're off the map.", 100),
      "ginger": ("You have the right general idea, but you're off the map.", 100),
      "purplecrayon": ("You're on the right track.", 133),
      "crayon": ("You're on the right track.", 133),
      "bluedanube": ("You're on the right track.", 130),
      "danube": ("You're on the right track.", 130),
      "tanuki": ("You're on the right track.", 130),
      "uki": ("You're on the right track.", 130),
      "greenapple": ("You're on the right track.", 130),
      "apple": ("You're on the right track.", 130),
      "reda": ("You're on the right track.", 130),
      "a": ("You're on the right track.", 130),
      "aquaspa": ("Right general idea, but you're off the map", 100),
      "spa": ("Right general idea, but you're off the map", 100),
      "goldstar": ("Right idea.", 100),
      "star": ("Right idea.", 100),
      "walgreens": ("Right idea", 100),
      "walgreen": ("Right idea, but you're missing an S", 100),
      "wals": ("Right idea", 100),
      "wal": ("Right idea, but you're missing an S", 100),
      "blackseatravel": ("Right idea, but you're off the map. And furthermore, that business is no longer there.", 100),
      "blacksea": ("Right idea, but you're off the map. And furthermore, that business is no longer there..", 100),
      "sea": ("Right idea, but you're off the map. And furthermore, that business is no longer there.", 100),
      "snowwhite": ("Right idea, but you're off the map.", 100),
      "snow": ("Right idea, but you're off the map.", 100),
      "cowboy": ("The mechanical horse at Toy Boat Cafe is for children only.<br>And furthermore has nothing to do with this puzzle, much like the word COWBOY.", 10),
      }
  },
  "offbeats" : {
    "instructions": "/s/offbeats.html",
    "solution": "calliope",
    "solved": "SLE/OVD",
    "hours": [],
    "hints": [
      (10, "The little phrases suggest 3-letter words."),
      (30, "The 3-letter words interlock to form 6-letter words."),
      (50, "The 6-letter words with the clues below them suggest new words"),
      (60, "The letters in parentheses spell the answer."),
      (70, "Some 3-letter words: AIR, LUG, ODD, and ROE."),
      (80, "Apiaries are neither for apes nor for birds."),
      (90, "Where was Leisure Suit Larry?"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "calliope": ("", 0),
      "aloeclip": ("Can you combine those words in a way you've done before?", 90),
      "clipaloe": ("Can you combine those words in a way you've done before?", 90),
      "clipclop": ("CLIP looks right.  In CLOP, the C and P look wrong.", 60),
      "clop": ("The LO looks right.  The C and P don't look right, though.", 50),
      "chipaloe": ("CHIP looks wrong but ALOE looks good.", 60),
      "caldcope": ("You combined CLCP and ADOE but I don't think those are words.", 60),
      "callcode": ("You combined CLCD and ALOE but I don't think CLCD is a word.", 60),
      "clalcope": ("You combined CACP and LLOE but I don't think those are words.", 60),
      "clip": ("Can you combine CLIP with something?", 90),
      "chip": ("That's mostly right, except the H.", 60),
      "aloe": ("Can you combine ALOE with something?", 90),
      "hoe": ("You're on the right track!", 10),
      "odd": ("You're on the right track!", 10),
      "one": ("You're on the right track!", 10),
      "lug": ("You're on the right track!", 10),
      "tee": ("You're on the right track!", 10),
      "eve": ("You're on the right track!", 10),
      "sit": ("You're on the right track!", 10),
      "ass": ("You're on the right track!", 10),
      "let": ("You're on the right track!", 10),
      "fun": ("You're on the right track!", 10),
      "fin": ("You're on the right track!", 10),
      "red": ("You're on the right track!", 10),
      "roe": ("You're on the right track!", 10),
      "oil": ("You're on the right track!", 10),
      "air": ("You're on the right track!", 10),
      "pay": ("You're on the right track!", 10),
      "hooded": ("You're on the right track!", 20),
      "lounge": ("You're on the right track!", 20),
      "teevee": ("You're on the right track!", 20),
      "tv": ("You're on the right track!", 20),
      "assist": ("You're on the right track!", 20),
      "fluent": ("You're on the right track!", 20),
      "friend": ("You're on the right track!", 20),
      "oriole": ("You're on the right track!", 20),
      "apiary": ("You're on the right track!", 20),
      "aviary": ("Double check that V.", 10),
      "cobra": ("You're on the right track!", 30),
      "lizard": ("You're on the right track!", 30),
      "television": ("You're on the right track!", 30),
      "help": ("You're on the right track!", 30),
      "language": ("You're on the right track!", 30),
      "pal": ("You're on the right track!", 30),
      "baltimore": ("You're on the right track!", 30),
      "bee": ("You're on the right track!", 30),
      "jay": ("No. Are you thinking of an aviary?<br>There's no aviary in this puzzle.", 1),
      }
  },
  "onestepbeyond" : {
    "instructions": "/s/onestepbeyond.html",
    "solution": "flower",
    "solved": "SOLVED",
    "hours": [], 
    "hints": [
      (0,  "If you're not in San Francisco and can't visit the puzzle site, guess SCOUT."),
      (1,  "The title is strange, but won't help you solve the puzzle.  It's just the name of a song that was on the 2 Tone Records label."),
      (10, "You are looking for the tiled steps where Moraga St would be."),
      (30, "The orientation of the picture on each domino is random."),
      (100, "Find the tiles whose pictures match your dominos."),
      (120, "Start arranging the dominos to match the steps' layout."),
      (130, "Some of the dominoes have a fine checkered pattern on them.  That should help you get started on the layout."),
      (131, "You're trying to use dominoes to tile a rectangle."),
      (132, "Your layout should look like a checkerboard. Unfortunately, there are many layouts that would look like a checkerboard."),
      (140, "If you know where <i>some</i> black squares belong but don't know what the white dots mean, maybe now is a good time to figure out what the white dots mean."),
      (160, "Numbered black domino squares obscure tiles with letters."),
      (190, "Use the black domino's number to index into the tile's letters."),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "flower": ("", 0),
      "scout": ("There are <a href=\"http://lahosken.san-francisco.ca.us/stuffs/2tone/steps/\">site photos</a> to look at.  Many, many photos.  Sorry about that.", 1),
      "lotus": ("Let's just call it a FLOWER, shall we?", 0),
      "lily": ("Let's just call it a FLOWER, shall we?", 0),
      "tulip": ("Let's just call it a FLOWER, shall we?", 0),
      "whatisrenustile": ("What is depicted on the tile that says \"RENU\"?", 190),
      "whatisrunestile": ("Did you swap a U and an E?", 185),
      "what": ("What is a nice word?  \"What\" is a nice word.", 100),
      "is": ("\"Is\" is a nice word.", 50),
      "renu": ("Renu is someone's name, according to a nice lady who does some landscaping by these stairs. \"Renu's\" is a nice word.", 160),
      "mcintyre": ("I guess you found the Moraga Steps OK.  Seems like you might be confused about what to do there, though.", 100),
      "renus": ("Renu is someone's name, according to a nice lady who does some landscaping by these stairs. \"Renu's\" is a nice word.", 160),
      "renua": ("That A looks wrong, but the rest looks correct: Renu'_.", 160),
      "reng": ("That G looks wrong, and you want another letter after the apostrophe.", 160),
      "eenus": ("That first E looks wrong, but the rest looks correct: _enu's.", 160),
      "venus": ("That V looks wrong, but the rest looks correct: _enu's.", 160),
      "wren": ("That W looks wrong, but you're on the right track: Ren_'_.", 160),
      "tile": ("\"Tile\" is a nice word.", 100),
      "anyword": ("I meant try any word that you think would fit into the blanks.", 1),
      }
  },
  "march" : {
    "instructions": "/s/march.html",
    "solution": "martin",
    "solved": "SOLVED",
    "hours": [2, 7, 8, 18, 19],
    "hints": [
      (0,  "If you're not in San Francisco and can't visit the puzzle site, guess SCOUT."),
      (10, "This is a 'photo walk'. Follow the photos to reach the destination.  From the site of the first photo, you can probably find the second... and so on."),
      (11, "Just because the photographer was facing in some direction, you don't need to walk in that direction."),
      (100, "Start at Union Square."),
      (120, "From Union Square, head South on Stockton past Neiman-Marcus.  The third photo shows the wall of Neiman-Marcus."),
      (130, "From Stockton Street, go East on O'Farrell.  The sixth photo shows the floor of the Adriano Goldschmied shop on O'Farrell."),
      (140, "Where O'Farrell meets Market St, cross Market.  The seventh photo shows a crosswalk across Market St."),
      (150, "From Market Street, take Yerba Buena Lane southeast.  The tenth photo shows a fountain you'll pass along the way."),
      (160, "In Yerba Buena Gardens, find the memorial under Samovar."),
      (180, "In the Memorial, find the translations on glass plaques."),
      (200, "For each language-number pair, use indexing to get one letter from the appropriate-language glass plaque"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "martin": ("", 0),
      "martinluther": ("", 0),
      "scout": ("The photo walk leads to a Martin Luther King memorial with some <a href=\"http://lahosken.san-francisco.ca.us/anecdotal/hunt/33.33/stelae/\">interesting inscriptions</a>.", 1),
      "hisfirstname": ("Who said these things?", 200),
      "stockton": ("That sounds promising.", 120),
      "adrianogoldschmied": ("Yep, that's their floor.", 130),
      "adriano": ("Yep, that's their floor.", 130),
      "goldschmied": ("Yep, that's their floor.", 130),
      "yerbabuena": ("Many things have that name. You might be on the right track, might not.", 0),
      "yerbabuenalane": ("That sounds promising.", 150),
      "yerbabuenagardens": ("That sounds promising.", 150),
      "memorial": ("That sounds promising.", 160),
      "martinlutherkingjrmemorial": ("That sounds promising.", 180),
      "martinlutherkingmemorial": ("That sounds promising.", 180),
      "kingmemorial": ("That sounds promising.", 170),
      "mlkmemorial": ("That sounds promising.", 180),
      "ybca": ("You're in the right general ara, but a little off track.", 0),
      "themarchhare": ("Wow, it fits the blanks nicely, doesn't it?<br>But it's not even close to the solution to this puzzle.", 10),
      }
  },
  "checkmate" : {
    "instructions": "/s/checkmate.html",
    "solution": "ebonyandivory",
    "solved": "S01V.D",
    "hours": [12, 13, 14, 16, 22, 23], 
    "hints": [
      (0,  "If you're not in San Francisco and can't visit the puzzle site, guess SCOUT."),
      (10, "'Checkmate' is a tile mural at 1351 Haight Street."),
      (20, "It's not so easy to \"read\" some parts of this mural.  For example, the piece in the first square of the third row from the bottom: let's say that's a black pawn.  As for the piece on the last square of the third row from the top... Could be a white pawn.  Could be a white bishop.  Fortunately, it's not used in this puzzle, so you don't need to figure out what it is."),
      (21, "You have instructions in Morse, five-bit binary, and Braille"),
      (100, "The instructions refer to pieces on the board."),
      (110, "When you decode the instructions, Black's turn is more straightforward than White's, right?  Maybe it's more straightforward to figure out what to do with those pieces."),
      (120, "White is checkmated, so we see the board from Black's point of view. To see the board from white's point of view, you'd turn it around."),
      (130, "You're solving for a phrase.  This phrase uses the letters D E I N O R S T U V W.  It uses some of those letters more than once."),
      (131, "You have instructions in Morse, five-bit binary, and Braille.  How have you used the \"Morse-ness\", \"binary-ness\", or \"Braille-ness\" of each instruction?"),
      (140, "The colors of the chessboard squares don't matter for this puzzle."),
      (150, "When looking for White's moves, turn the board upside down."),
      (160, "The five-bit binary instructions lists pieces you can find in a horizontal row of five squares (with no other pieces in that five-square span)."),
      (170, "Each Morse instruction lists pieces you can find in a horizontal row.  Maybe not <i>all</i> of the pieces in that row."),
      (180, "Each Braille instruction lists pieces you can find in a 2x3 rectangle"),
      (190, "If a square with a piece is 1 and an empty square is 0, a set of 5 squares can make a 5-bit binary letter"),
      (200, "If a row of 3 pieces is a dash and a 'lone' piece is a dot, then a chessboard row can make a Morse letter"),
      (210, "If a piece is a 'bump' and an empty square is 'flat', then a 2x3 section of chessboard could make a Braille letter"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "ebonyandivory": ("", 0),
      "ebonyivory": ("", 0),
      "scout": ("The site is a mosaic mural.  Here are some <a href=\"https://lahosken.san-francisco.ca.us/stuffs/2tone/checkmate/\">photos</a>.  Some photos have the color tweaked to make it easier to see details.", 1),
      "duetsteviewonder": ("Can you name a Stevie Wonder duet?", 150),
      "steviewonderduet": ("Can you name a Stevie Wonder duet?", 150),
      "steviewonder": ("That sounds promising.", 100),
      "stevie": ("That sounds promising.", 100),
      "wonder": ("That sounds promising.", 100),
      "duet": ("That sounds promising.", 100),
      "ebony": ("Ebony and _____", 200),
      "ivory": ("_____ and Ivory", 200),
      "itsyou": ("A different duet, with Paul McCartney.", 200),
      "wearetheworld": ("Think more about a duet with Paul McCartney.", 200),
      "justgoodfriends": ("A different duet, with Paul McCartney.", 200),
      "getit": ("A different duet, with Paul McCartney.", 200),
      "mylove": ("A different duet, with Paul McCartney.", 200),
      "atimetolove": ("A different duet, with Paul McCartney.", 200),
      "amore": ("A different duet, with Paul McCartney.", 200),
      }
  },
  "dualtone" : {
    "instructions": "/s/dual.html",
    "solution": "item",
    "solved": "6OL4ED",
    "hours": [1, 4, 6, 11, 15, 20, 21],
    "hints": [
      (0,  "If you're not in San Francisco and can't visit the puzzle site, guess SCOUT."),
      (5, "The Ferry Terminal is behind the Ferry Building.<br>You can \"guess an answer\" with a payphone number to tell the hint system you've arrived."),
      (100, "The payphones are at the South end of the Stephan Leonoudakis Golden Gate Ferry terminal building, near the Ferry Building"),
      (110, "Suppose the first column had \"frequency\" 5. You know the total frequency for the <b>1</b> button.  What is the first row's frequency?"),
      (112, "There's no reason to think that the first column's frequency is 5, but whatever value you choose, you'll get the same final answer."),
      (120, "If the first column is 5, the first row is 4."),
      (130, "If the first row is 4 and you know the frequency of the <b>3</b> button, what is the third column's frequency?"),
      (140, "If the first row is 4, then the third column is 20."),
      (150, "If the third column is 20, and you know the frequencies of the 6 and 9 buttons, what are the frequencies of the second and third rows?"),
      (160, "What are the frequencies of the 4, 5, 7, and 8 buttons? Can you decode the 21 24 6 9 24 12 14 now?"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "item": ("", 0),
      "promenade": ("", 0), # old answer
      "skew": ("", 0), # old answer
      "scout": ("Those pay phone numbers are 362 9099 and 362 9102.  Later on, my voice will tell you to enter a guess.  When that happens, enter what you think the <i>next</i> part is to get more 'scout' info.  E.g., if I tell you to enter CHAPTER TWO then you should enter CHAPTER THREE instead.", 1),
      "partc": ("If you follow the instructions in PART B, you eventually find the word ITEM.", 1),
      "partb": ("Take a look at <a href=\"/s/dual_b.html\">Part B</a>.", 190),
      "9371354": ("Looks like a telephone number.  You should probably call that number.", 160),
      "4159371354": ("Looks like a telephone number.  You should probably call that number.", 160),
      "14159371354": ("Looks like a telephone number.", 160),
      "dialect": ("You figured out the grid.  Call the number.", 150),
      "3629099": ("Yay, you found the pay phones.", 100),
      "3629102": ("Yay, you found the pay phones.", 100),
      "4153629099": ("Yay, you found the pay phones.", 100),
      "4153629102": ("Yay, you found the pay phones.", 100),
      "14153629099": ("Yay, you found the pay phones.", 100),
      "14153629102": ("Yay, you found the pay phones.", 100),
      "parta": ("Take a look at <a href=\"/s/dual.html\">Part A</a>.", 0),
      },
  },
  "teuton" : {
    "instructions": "/s/teuton.html",
    "solution": "zeit",
    "solved": "GEL&Ouml;ST",
    "hours": [],
    "hints": [
      (10, "Each fake German phrase translates to a common English saying."),
      (20, "The internet has some automatic translation services."),
      (40, "Use one letter from each English saying to form more German."),
      (100, "You almost formed a common German phrase.  Can you Google it?"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "zeit": ("", 0),
      "kommtblankkommtrat": ("Fill in the blank in this German proverb.", 100),
      "kommtblankommtrat": ("Fill in the blank in this German proverb.", 100),
      "kommt": ("That sounds good", 40),
      "kommi": ("That sounds close, but not quite right.  Maybe check for gender-neutral canines?", 35),
      "kommiblankommtrat": ("Wow, close, but one word looks wrong.  You can \"check\" parts of your answer by entering one word at a time.", 100),
      "candy": ("CANDY sounds like a translation of KOMMI, which is close, but not quite right.  Maybe check for gender-neutral canines?", 30),
      "comes": ("COMES sounds like a translation of KOMMT, which is right.", 30),
      "kammt": ("That sounds close, but not quite right.  The A looks wrong", 35),
      "kemmt": ("That sounds close, but not quite right.  The E looks wrong", 35),
      "kummt": ("That sounds close, but not quite right.  The U looks wrong", 35),
      "blank": ("That sounds good", 40),
      "rat": ("That sounds good", 40),
      "ras": ("That sounds close, but not quite right.  Double-check that last letter.", 40),
      "rag": ("That sounds close, but not quite right.  Double-check that last letter.", 40),
      "time": ("Leave it in Deutsche, please.", 100),
      "times": ("Leave it in Deutsche, please.", 100),
      "day": ("Leave it in Deutsche, please.", 100),
      "theday": ("Leave it in Deutsche, please.", 100),
      }
  },
  "rebus" : {
    "instructions": "/s/rebus.html",
    "solution": "hearty",
    "solved": "S+olive-I+D",
    "hours": [],
    "hints": [
      (10, """Solve the rebus puzzles. If you don't know what a rebus is, you might want to visit the <a href="/about/game">About</a> screen."""),
      (30, "mouse + r = MOUSER. finch + l = FLINCH. t + hatch = THATCH"),
      (35, "That decoration at the top of the instructions page is strange."),
      (40, "If you don't know what else to do with something strange, you might count things."),
      (41, "The decoration at the top has 6 horizontal rows.  The long grid has 6 columns.   That's a coincidence."),
      (42, "The decoration at the top has 6 horizontal rows.  There are six blanks at the bottom.   That's a coincidence."),
      (50, "That decoration at the top of the instructions page has 26 squares."),
      (51, "The alphabet has 26 letters."),
      (60, "Write the alphabet in the decoration."),
      (65, "In the decoration, what color is B?"),
      (70, "The grid at the top of the instructions tells you that A is white, B is black, etc."),
      (90, "Fill in the big grid with black and white, using the color suggestions from the decoration."),
      (100, "Turn the big grid on its side to see another rebus"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "hearty": ("", 0),
      "y": ("Maybe you could rebus-ly combine that with a picture?", 60),
      "plus": ("Maybe you could rebus-ly combine things?", 10),
      "heart": ("Maybe you could rebus-ly combine that with a letter?", 60),
      "nation": ("You're on the right track!", 10),
      "triple": ("You're on the right track!", 20),
      "riddle": ("You're on the right track!", 15),
      "batter": ("You're on the right track!", 15),
      "curate": ("You're on the right track!", 15),
      "thatch": ("You're on the right track!", 15),
      "cleave": ("You're on the right track!", 15),
      "dragon": ("You're on the right track!", 15),
      "attach": ("You're on the right track!", 15),
      "fairly": ("You're on the right track!", 15),
      "flinch": ("You're on the right track!", 20),
      "choral": ("You're on the right track!", 20),
      "mouser": ("You're on the right track!", 20),
      "pliant": ("You're on the right track!", 15),
      "preach": ("You're on the right track!", 15),
      "handle": ("You're on the right track!", 15),
      "carpet": ("You're on the right track!", 15),
      "weight": ("You're on the right track!", 15),
      "cranky": ("You're on the right track!", 15),
      "palate": ("You're on the right track!", 15),
      },
  },
  "crossword" : {
    "instructions": "/s/crossword.html",
    "solution": "doublecheck",
    "solved": """SO<img src="/s/crossword_bk.png">V<img src="/s/crossword_wh.png">D""",
    "hours": [],
    "hints": [
      (1,  "There's no shame looking up crossword answers on the internet if you have to work so hard to figure out what the clues are."),
      (2,  "It's a crossword puzzle, with some letters replaced by black or white rectangles."),
      (10, "The letters in \"black\" are B L A C K"),
      (15, "The letters in \"white\" are W H I T E"),
      (30, "All letters in the grid are from \"black\" or \"white\"."),
      (35, "The circled letters form a two-word phrase."),
      (40, "A cel is a transparent sheet."),
      (45, "You might say that a person who is baked is a \"bakee\"."),
      (50, "Khaki is a kind of fabric"),
      (55, "Tee-Ball is like baseball without the pitching."),
      (60, "Some grid squares are circled.  What do they spell?"),
      (65, "\"Too Much Pressure\" is a song title."),
      (70, "Bill Hewlett co-founded a Silicon Valley startup."),
      (75, "Bill Belichick is a football coach."),
      (80, "Elmer Fudd sings \"Kill the Wabbit\" to the tune of Wagner's \"Ride of the Valkyries\" in the cartoon \"What's Opera, Doc?\""),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response" : {
      "doublecheck" : ("", 0),
      "checkcheck" : ("What phrase does that suggest to you?", 60),
      "soundcheck" : ("Maybe more like d_____ check?", 60),
      "plaid" : ("Maybe more like d_____ check?", 60),
      "testing" : ("Maybe more like d_____ check?", 60),
      "checksquared" : ("Maybe more like d_____ check?", 60),
      "twochecks" : ("Maybe more like d_____ check?", 60),
      "onetwocheck" : ("Maybe more like d_____ check?", 60),
      "checkonetwo" : ("Maybe more like d_____ check?", 60),
      "onetwo" : ("Maybe more like d_____ check?", 60),
      "onetwothree" : ("Maybe more like d_____ check?", 60),
      "12" : ("Maybe more like d_____ check?", 60),
      "123" : ("Maybe more like d_____ check?", 60),
      "testing123" : ("Maybe more like d_____ check?", 60),
      "checkcheckmate" : ("Maybe more like d_____ check?", 60),
      "testingonetwothree" : ("Maybe more like d_____ check?", 60),
      "checkplease" : ("Maybe more like d_____ check?", 60),
      "checkthemic" : ("Maybe more like d_____ check?", 60),
      "checkthemike" : ("Maybe more like d_____ check?", 60),
      "miccheck" : ("Maybe more like d_____ check?", 60),
      "mikecheck" : ("Maybe more like d_____ check?", 60),
      "volumecheck" : ("Maybe more like d_____ check?", 60),
      "isthisthingon" : ("Maybe more like d_____ check?", 60),
      "checkitout" : ("Maybe more like d_____ check?", 60),
      "checkerboard" : ("Maybe more like d_____ check?", 60),
      "chessboard" : ("Maybe more like d_____ check?", 60),
      "teeball" : ("That's in the puzzle", 20),
      "caltech" : ("That's in the puzzle", 20),
      "tabac" : ("That's in the puzzle", 20),
      "tabak" : ("Careful, it's not spelled quite like that", 20),
      "kalel" : ("That's in the puzzle", 20),
      "calel" : ("Careful, it's not spelled quite like that.", 20),
      "killthewabbit" : ("That's in the puzzle", 20),
      "billbelichick" : ("That's in the puzzle", 20),
      "billbelicheck" : ("Careful, it's not spelled quite like that", 20),
      "belicheck" : ("Careful, it's not spelled quite like that", 20),
      "cecil" : ("That's in the puzzle", 20),
      "title" : ("That's in the puzzle", 20),
      "letitbe" : ("That's in the puzzle", 20),
      "hewlett" : ("Thank you, Bill and Dave. That's in the puzzle.", 20),
      "thebeeb" : ("That's in the puzzle", 20),
      "cel" : ("That's in the puzzle", 20),
      "tikal" : ("That's in the puzzle", 20),
      "label" : ("That's in the puzzle", 20),
      "licit" : ("That's in the puzzle", 20),
      "latke" : ("That's in the puzzle", 20),
      "latka" : ("Careful, it's not spelled quite like that.", 20),
      "chile" : ("That's in the puzzle", 20),
      "chili" : ("Careful, it's not spelled quite like that.", 20),
      "catch" : ("That's in the puzzle", 20),
      "khaki" : ("That's in the puzzle", 20),
      "bakee" : ("That's in the puzzle.  Pretty contrived, eh?", 20),
      "hal" :   ("Affirmative, Dave, I read you.  That's in the puzzle.", 20),
      "thebeat" : ("That's in the puzzle", 20),
      "blackwhite" : ("BLACK squares are black. WHITE squares are white.", 20),
      "whiteblack" : ("BLACK squares are black. WHITE squares are white.", 20),
      "black" : ("BLACK squares are black. WHITE squares are white.", 20),
      "white" : ("BLACK squares are black. WHITE squares are white.", 20),
      }
  },
  "sudoku" : {
    "instructions": "/s/sudoku.html",
    "solution": "onjuku",
    "solved": "SOLV54",
    "hours": [],
    "hints": [
      (10, "Fill in the grid as best you can."),
      (11, "Beware of automatic sudoku solvers here.  If you don't figure out the trick to this sudoku, you'll think there's more than one way to solve it.  An 'automatic' solver might not find the <i>right</i> way to solve this one."),
      (30, "Those numbers printed off to the side will be in white squares."),
      (60, "The top row of the grid should be 327 169 584."),
      (70, "If 1=A, 2=B, 3=C, ... what are the top five squares of the first column?"),
      (80, "If 1=A, 2=B, 3=C, ... what are the bottom five squares of the last column?"),
      (100, "If you want tourist information, you might search the internet."),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
      ],
    "response": {
      "onjuku": ("", 0),
      "onjukubeach": ("", 0),
      "onjuko": ("OnjukU is a nice beach.  Try OnjukU.  With a \"U\" at the end.", 0),
      "torami": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "makuhari": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "kujukuri": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "hama": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "kujukurihama": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "shirosaki": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "wadaura": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "boso": ("Onjuku is a nice beach.  Try Onjuku.", 100),
      "inage": ("First artificial beach in Japan, very impressive.  But not the answer.  Onjuku is a nice beach.  Try Onjuku.", 100),
      "inagebeach": ("First artificial beach in Japan, very impressive.  But not the answer.  Onjuku is a nice beach.  Try Onjuku.", 100),
      "chibabeach": ("Can you name a beach in Chiba prefecture?", 90),
      "chinabeach": ("That \"N\" looks funny.", 85),
      "chiubeach":  ("That \"U\" looks funny.", 85),
      "chiba": ("Don't forget the letters in the rightmost column.", 70),
      "beach": ("Don't forget the letters in the leftmost column.", 60),
      "3892125138": ("Awesome.  Can you turn that into a five-letter place name and a five-letter word?", 50),
      "38921": ("Awesome.  Can you turn that into a five-letter place name?", 50),
      "25138": ("Awesome.  Can you turn that into a five-letter word?", 50),
      "2156345162": ("Maybe you found something that would be a solution to an ordinary Sudoku.  But your solution doesn't fit those little numbers printed around the side of the puzzle.  Maybe you should figure out what those mean.", 15),
      "2156326154": ("Maybe you found something that would be a solution to an ordinary Sudoku.  But your solution doesn't fit those little numbers printed around the side of the puzzle.  Maybe you should figure out what those mean.", 15),
      }
  },
  "meta": {
    "instructions": "/s/ofcoursetheresameta.html",
    "solution": "summarize",
    "solved": """SOLVED""",
    "hours": [69], # never occurs; makes algo hesitate to unlock it
    "hints": [
            (10, "The rebus at the top just solves to 'Meta'"),
            (20, "The mixture of German and numbers tells you that this is a Metapuzzle.  That means you'll be putting together things that you've learned in a reduced form."),
            (130, "After the 'Meta' and the German+numbers text comes the main part of the puzzle.  Each piece of this main part solves to a NATO phonetic letter"),
      (10000, """Send your question to <a href="mailto:roughneck@2tonegame.org">roughneck@2tonegame.org</a>"""),
            ],
    "response": {
      "summarize": ("", 0),
      "submarine": ("Wow, you're really close, just a couple of letters off.", 120),
      "sierra": ("That sounds like a promising beginning.", 100),
      "uniform": ("That sounds good.", 100),
      "mike": ("That sounds good.", 100),
      "alfaromeo": ("That sounds good. That sounds good.", 100),
      "alfa": ("ALFA _____", 90),
      "romeo": ("____ ROMEO", 100),
      "india": ("That sounds good.", 100),
      "zulu": ("That sounds good.", 100),
      "echo": ("That sounds like a good ending.", 100),
            }
        }
}

def PickNextActs(player, now):
  # remember that "meta" has an hour that never comes
  retval = []
  candidates = []
  lastditch = []
  unlocked_count = 0
  for a in SUGGESTED_ORDER:
    status = player.GetActStatus(a)
    if status:
      if status[1] == "unlocked":
        unlocked_count += 1
      continue
    if now.hour in ACT_DATA[a]["hours"]: 
      retval.append(a)
      continue
    if ACT_DATA[a]["hours"]:
      lastditch.append(a)
    else:
      candidates.append(a)
  candidates.extend(lastditch)
  if unlocked_count < 1: retval.extend(candidates[:1])
  if unlocked_count == 1 and not retval: retval.extend(candidates[:1])
  return retval
