from google.appengine.ext import db
import datetime

def PackActStatus(act, state, highest, points, last_updated_utcdatetime):
    highest_s = str(highest)
    points_s = str(points)
    date_s = last_updated_utcdatetime.strftime('%Y/%m/%d %H:%M:%S')
    return '%s\t%s\t%s\t%s\t%s' % (act, state, highest_s, points_s, date_s)

def UnpackActStatus(s):
    (act, state, highest_s, points_s, date_s) = s.split("\t")
    date = datetime.datetime.strptime(date_s, '%Y/%m/%d %H:%M:%S')
    return (act, state, int(highest_s), int(points_s), date)

class Player:
    def __init__(self):
        db_model = None
        openid_id = ""
        act_status = []

    def LoadFromDBByID(self, openid_id):
        models = PlayerModel.all()
        models.filter("openid_id =", openid_id)
        models_l = models.fetch(1)
        if models_l:
            self.db_model = models_l[0]
            self.openid_id = self.db_model.openid_id
            self.act_status = self.db_model.act_status
            self.recent = self.db_model.recent
            self.score = self.db_model.score
            self.Commit()
        else:
            # new player!
            self.db_model = PlayerModel(openid_id=openid_id)
            self.openid_id = self.db_model.openid_id
            self.act_status = self.db_model.act_status
            self.recent = self.db_model.recent
            self.score = 0
            self.ProvisionNewPlayer()
            self.Commit()

    def TouchRecent(self):
        self.recent = datetime.datetime.now()

    def ProvisionNewPlayer(self):
        self.UnlockAct("example")
        self.score = 500

    def Commit(self):
        if (self.db_model):
            self.db_model.recent = self.recent
            self.db_model.score = self.score
            self.db_model.put()

    def UnlockAct(self, act):
        self.recent = datetime.datetime.now()
        for i in self.act_status:
            if i.startswith(act + "\tlocked"): 
                self.act_status.remove(i)
                break
            if i.startswith(act + "\t"):
                return # already unlocked?  if so, do nothing
        self.act_status.append(PackActStatus(act, "unlocked", 0, 0, 
                                             datetime.datetime.utcnow()))
        self.score += 100

    def MarkSolved(self, act):
        self.recent = datetime.datetime.now()
        # REMIND like UnlockAct, I want to get and delete old status
        for i in self.act_status:
            (a, state, highest, points, date) = UnpackActStatus(i)
            if a == act:
                self.act_status.remove(i)
                break
        else:
            return
        self.act_status.append(PackActStatus(act, "solved", highest, 
                                             points, self.recent))


    def PurchaseHint(self, act, cost):
        self.recent = datetime.datetime.now()
        # REMIND like UnlockAct, I want to get and delete old status
        for i in self.act_status:
          (a, state, highest, points, date) = UnpackActStatus(i)
          if a == act:
              break
        else:
            return
        if highest >= cost: 
          return
        minutes_since_update = ((self.recent - date).seconds / 60) + 1
        net_cost = cost - (points + minutes_since_update)
        highest = cost
        if net_cost > 0:
            self.score -= net_cost
            points = cost
        else:
            points += minutes_since_update
        self.act_status.remove(i)
        self.act_status.append(PackActStatus(act, state, highest, 
                                             points, self.recent))

    def BoostHint(self, act, newmin):
        """Player guessed something that shows they don't need a hint."""
        self.recent = datetime.datetime.now()
        # REMIND like UnlockAct, I want to get and delete old status
        for i in self.act_status:
          (a, state, highest, points, date) = UnpackActStatus(i)
          if a == act:
              break
        else:
            return
        if highest >= newmin: return
        minutes_since_update = ((self.recent - date).seconds / 60) + 1
        if (points + minutes_since_update) >= newmin: return
        self.act_status.remove(i)
        self.act_status.append(PackActStatus(act, state, highest, 
                                             newmin, self.recent))

    def GetActStatus(self, act):
        for i in self.act_status:
            if i.startswith(act + "\t"):
                return UnpackActStatus(i)
        return []

class PlayerModel(db.Model):
    openid_id = db.StringProperty()
    act_status = db.StringListProperty()
    recent = db.DateTimeProperty(auto_now_add=True)
    created = db.DateTimeProperty(auto_now_add=True)
    score = db.IntegerProperty()

def RecentStats():
    models = PlayerModel.all()
    models.order("-recent")
    models.fetch(30)
    retval = []
    for model in models[:30]:
        retval.append({"id": model.openid_id,
                       "status": "\n".join(model.act_status),
                       "score": model.score })
    return retval
